`timescale 1ns / 1ps

module codec_if(
   input  clk,
   input  rst,
   
   output adc_mclk,
   output adc_bclk,
   output adc_lrclk,
   input  adc_din,

   input         adc_l_rd,
   output        adc_l_empty,
   output [23:0] adc_l_data,
   input         adc_r_rd,
   output        adc_r_empty,
   output [23:0] adc_r_data,

   input         dac_l_wr,
   input  [23:0] dac_l_data,
   output        dac_l_full,
   input         dac_r_wr,
   input  [23:0] dac_r_data,
   output        dac_r_full,
   
   output dac_mclk,
   output dac_bclk,
   output dac_lrclk,
   output dac_dout
);


reg [9:0] clk_div = 0;
always @ (posedge clk)
if (rst)
   clk_div <= 0;
else
   clk_div <= clk_div + 1;
   
assign adc_mclk = clk_div[1];
assign dac_mclk = clk_div[1];

assign adc_bclk = clk_div[3];
assign dac_bclk = clk_div[3];

assign adc_lrclk = clk_div[9];
assign dac_lrclk = clk_div[9];


wire [4:0] adc_bcnt;
assign adc_bcnt = clk_div[8:4];

wire [4:0] dac_bcnt;
assign dac_bcnt = clk_div[8:4];

wire bclk_rise;
assign bclk_rise = (clk_div[3:0]==4'b0111);
wire bclk_fall;
assign bclk_fall = (clk_div[3:0]==4'b1111);


reg [23:0] adc_shr;
always @ (posedge clk)
if (bclk_rise==1)
   adc_shr <= {adc_shr[22:0], adc_din};
   
reg adc_valid_l = 0;
reg adc_valid_r = 0;
always @ (posedge clk)
if (rst)
begin
   adc_valid_l <= 0;
   adc_valid_r <= 0;
end
else
begin
   if (bclk_rise==1 & adc_bcnt==31 & clk_div[9]==1)
      adc_valid_l <= 1;
   else
      adc_valid_l <= 0;

   if (bclk_rise==1 & adc_bcnt==31 & clk_div[9]==0)
      adc_valid_r <= 1;
   else
      adc_valid_r <= 0;
end

srl_fifo adc_srl_fifo_l(
   .clk(clk),
   .rst(rst),
   .xwr(adc_valid_l),
   .xrd(adc_l_rd),
   .din(adc_shr),
   .dout(adc_l_data),
   .empty(adc_l_empty),
   .full(),
   .dcnt()
);

srl_fifo adc_srl_fifo_r(
   .clk(clk),
   .rst(rst),
   .xwr(adc_valid_r),
   .xrd(adc_r_rd),
   .din(adc_shr),
   .dout(adc_r_data),
   .empty(adc_r_empty),
   .full(),
   .dcnt()
);





wire [4:0] fifo_dcnt_l, fifo_dcnt_r;
wire dac_lfifo_empty;
wire dac_lfifo_rd;
wire [23:0] dac_lfifo_dout;
srl_fifo dac_srl_fifo_l(
   .clk(clk),
   .rst(rst),
   .xwr(dac_l_wr),
   .xrd(dac_lfifo_rd),
   .din(dac_l_data),
   .dout(dac_lfifo_dout),
   .empty(dac_lfifo_empty),
   .full(dac_l_full),
   .dcnt(fifo_dcnt_l)
);

wire dac_rfifo_empty;
wire dac_rfifo_rd;
wire [23:0] dac_rfifo_dout;
srl_fifo dac_srl_fifo_r(
   .clk(clk),
   .rst(rst),
   .xwr(dac_r_wr),
   .xrd(dac_rfifo_rd),
   .din(dac_r_data),
   .dout(dac_rfifo_dout),
   .empty(dac_rfifo_empty),
   .full(dac_r_full),
   .dcnt(fifo_dcnt_r)
);

reg fifo_rd_en;
always @ (posedge clk)
if (rst | dac_rfifo_empty | dac_lfifo_empty)
   fifo_rd_en <= 0;
else if ((fifo_dcnt_l>2) & (fifo_dcnt_r>2))
   fifo_rd_en <= 1;


reg [31:0] dac_shr;
always @ (posedge clk)
if (bclk_fall==1)
   if (dac_bcnt==31)
      if (fifo_rd_en==0)
         dac_shr <= 0;
      else if (clk_div[9]==0)
         dac_shr <= {8'b0, dac_lfifo_dout};
      else
         dac_shr <= {8'b0, dac_rfifo_dout};
   else
      dac_shr <= {dac_shr[30:0], 1'b0};
      
assign dac_lfifo_rd = (fifo_rd_en==1 & dac_lfifo_empty==0 & bclk_fall==1 & dac_bcnt==31 & clk_div[9]==0);
assign dac_rfifo_rd = (fifo_rd_en==1 & dac_rfifo_empty==0 & bclk_fall==1 & dac_bcnt==31 & clk_div[9]==1);

assign dac_dout = dac_shr[31];

endmodule
