`timescale 1ns / 1ps

////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:   14:05:46 05/10/2014
// Design Name:   fir_hw
// Module Name:   D:/xxx_hls/_src/tb_fir.v
// Project Name:  hls_fir
// Target Device:  
// Tool versions:  
// Description: 
//
// Verilog Test Fixture created by ISE for module: fir_hw
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////

module tb_fir;

	// Inputs
	reg clk;
	reg rst;
	wire [17:0] input_l_V;
	wire input_l_V_ap_vld;
	wire [17:0] input_r_V;
	wire input_r_V_ap_vld;
	wire res_l_V_ap_ack;
	wire res_r_V_ap_ack;

	// Outputs
	wire input_l_V_ap_ack;
	wire input_r_V_ap_ack;
	wire [17:0] res_l_V;
	wire res_l_V_ap_vld;
	wire [17:0] res_r_V;
	wire res_r_V_ap_vld;

	initial begin
		// Initialize Inputs
      clk = 1;
      rst = 1;
		// Add stimulus here
      #102 rst = 0;
	end
   
   
always #5 clk <= ~clk;

reg [17:0] cntr = 0;
always @ (posedge clk)
if (adc_l_full==0 & adc_r_full==0)
   cntr <= cntr + 1;


wire adc_valid_l, adc_valid_r;
wire adc_l_rd, adc_r_rd;
wire [23:0] adc_shr;
wire [23:0] adc_l_data, adc_r_data;
wire adc_l_empty, adc_r_empty;
wire adc_l_full, adc_r_full;

assign adc_l_wr = ~adc_l_full & ~adc_r_full;
assign adc_r_wr = ~adc_l_full & ~adc_r_full;
assign adc_shr = {cntr, 6'b0};


srl_fifo adc_srl_fifo_l(
   .clk(clk),
   .rst(rst),
   .wr(adc_l_wr),
   .rd(adc_l_rd),
   .din(adc_shr),
   .dout(adc_l_data),
   .empty(adc_l_empty),
   .full(adc_l_full),
   .dcnt()
);

srl_fifo adc_srl_fifo_r(
   .clk(clk),
   .rst(rst),
   .wr(adc_r_wr),
   .rd(adc_r_rd),
   .din(adc_shr),
   .dout(adc_r_data),
   .empty(adc_r_empty),
   .full(adc_r_full),
   .dcnt()
);

assign input_l_V_ap_vld = ~adc_l_empty;
assign input_r_V_ap_vld = ~adc_r_empty;
assign adc_l_rd = ~adc_l_empty & input_l_V_ap_ack;
assign adc_r_rd = ~adc_r_empty & input_r_V_ap_ack;

// Instantiate the Unit Under Test (UUT)
fir_hw uut (
   .ap_clk(clk), 
   .ap_rst(rst), 
   .input_l_V(adc_l_data[23:6]), 
   .input_l_V_ap_vld(input_l_V_ap_vld), 
   .input_l_V_ap_ack(input_l_V_ap_ack), 
   .input_r_V(adc_r_data[23:6]), 
   .input_r_V_ap_vld(input_r_V_ap_vld), 
   .input_r_V_ap_ack(input_r_V_ap_ack), 
   .res_l_V(res_l_V), 
   .res_l_V_ap_vld(res_l_V_ap_vld), 
   .res_l_V_ap_ack((en==0)), 
   .res_r_V(res_r_V), 
   .res_r_V_ap_vld(res_r_V_ap_vld), 
   .res_r_V_ap_ack((en==0))
);

reg [7:0] en=0;
always @ (posedge clk)
   en <= en + 1;

reg [17:0] res_l, res_r;
always @ (posedge clk)
if (res_l_V_ap_vld)
   res_l <= res_l_V;

always @ (posedge clk)
if (res_r_V_ap_vld)
   res_r <= res_r_V;

endmodule

